-- soft-nrg Development GmbH
-- Version 1.1 / Stand: 10.10.2014
-- Beschreibung: Amtsvorwahl 0 wird bei Telefonnummern mit mehr als drei Ziffern hinzugefuegt, bei den Notrufnummern 110 und 112 wird die AMTSVORWAHL auch hinzugefuegt.

-- 'Enum'-s:
-- 'Call'-specific
local CallState =
{
  Idle = 0;         -- The call has been created, but it's not yet connected. A call can never transition into the idle state. This is the initial state for both incoming and outgoing calls.
  Inprogress = 1;   -- The service provider is working on making a connection. This state is valid only on outgoing calls. This message is optional, because a service provider may have a call transition directly to the connected state.
  Connected = 2;    -- Call has been connected to the remote end and communication can take place.
  Disconnected = 3; -- Call has been disconnected. There are several causes for disconnection. See the table of valid call state transitions below.
  Offering = 4;     -- A new call has appeared, and is being offered to an application (incoming call).
  Hold = 5;         -- The call is in the hold state.
  Queued = 6;       -- The call is queued.
};
local CallPrivilege =
{
  Monitor = 0;      -- The application is a monitor of the call (call control is not possible).
  Owner = 1;        -- The application is the owner of the call (call control is possible).
};
local CallOrigin =
{
  Conference = 0;   -- The call is a conference call, that is, it is the application's connection to the conference bridge in the switch.
  External = 1;     -- The call originated as an incoming call on an external line.
  Inbound = 2;      -- The call originated as an incoming call, but the service provider is unable to determine whether it came from another station on the same switch or from an external line.
  Internal = 3;     -- The call originated as an incoming call at a station internal to the same switching environment.
  Outbound = 4;     -- The call originated from this station as an outgoing call.
  Unavailable = 5;  -- The call origin is not available and will never become known for this call.
  Unknown = 6;      -- The call origin is currently unknown but may become known later.
};

-- 'Address'-specific
local AddressState =
{
  InService = 0;    -- Normal state; the address can be used.
  OutOfService = 1; -- The address is temporarily out of service, but may go back into service at some time.
};
local CallFeature =         -- flags
{
  None          = 0x0001;   -- No known feature
  Accept	= 0x0001;   -- Can accept calls.
  Answer	= 0x0002;   -- Can answer calls.
  Dial          = 0x0004;   -- Can dial calls.
  Drop          = 0x0008;   -- Can drop calls.
  Hold          = 0x0010;   -- Can hold calls.
  Redirect      = 0x0020;   -- Can redirect calls.
  Transfer      = 0x0040;   -- Can transfer calls with consultation.
  BlindTransfer = 0x0080;   -- Can transfer calls without consultation.
};

-- interface: functions exposed to be called from C
function createAddressData()
  return
    {
      identifier = -1;
      state = AddressState.InService;
      callFeatures = CallFeature.None;
      hasTerminalSupport = false;
      name = "";
      serviceProviderName = "";
      dialableAddress = "";
      permanentAddressId = "";
    };
end

local function isExternalNumber(number)
  local number = tostring(number);
  if (string.len(number) > 3) then return true; end;

  local specialShortExternals = { '110'; '112'; };  -- DE Police (110) and European Emergency (112)
  for _, v in ipairs(specialShortExternals) do
    if (number == v) then return true; end;
  end;

  return false;
end

local function callDataBasedOrigin(callData)
  -- call state based logic
  if (callData.call.state == CallState.Offering) then
    return CallOrigin.Inbound;
  end;
  if (callData.call.state == CallState.Inprogress) then
    return CallOrigin.Outbound; 
  end;
  -- call data based logic, the call state is not enough
  if (callData.address.name == callData.call.calledIdName) then
    return CallOrigin.Inbound;
  end;
  return CallOrigin.Outbound;
end

-- 'Constructor' functions
function createCallData()
  return
  {
    -- call related fields
    call =
    {
      identifier = -1;	        -- the ID of this call
      addressIdentifier = -1;   -- the ID of this call's address
      tapiCallIdentifier = -1;  -- the TAPI ID of this call (could be the same for multiple calls if they belong to the same conversation)
      state = CallState.Idle;
      privilege = CallPrivilege.Monitor;
      origin = CallOrigin.Unknown;
      callerIdName = "";
      callerIdNumber = "";
      calledIdName = "";
      calledIdNumber = "";
      connectedIdName = "";
      connectedIdNumber = "";
      redirectionIdName = "";
      redirectionIdNumber = "";
      redirectingIdName = "";
      redirectingIdNumber = "";
    };
    -- address related fields
    address = createAddressData();
  };
end

-- the addressData.identifier is read only that means that the caller will ignore that value from the returned result
function patchAddressData(addressData)
  --~ if ("table" ~= type(addressData)) then
    --~ return nil, string.format("addressData to process is not table (it's [%s])!", type(addressData));
  --~ end;

  local dataChanged = false;
  -- change fields here if necessary (and set dataChanged if so)
  --

  return dataChanged, addressData;
end

-- callData.call.identifier, callData.call.addressIdentifier and callData.address.* fields
-- are read only that means that the caller will ignore those values from the returned result
function patchCallData(callData)
  if ("table" ~= type(callData)) then
    return nil, string.format("callData to process is not table (it's [%s])!", type(callData));
  end;

  local dataChanged = false;
  -- change fields here if necessary (and set dataChanged if so)
  --
  -- to external outgoing call
  if (isExternalNumber(callData.call.calledIdNumber)) then
    callData.call.calledIdNumber = string.sub(callData.call.calledIdNumber, 2, -1);
    dataChanged = true;
  end;
  -- from external incomming call
  if (isExternalNumber(callData.call.callerIdNumber)) then
    callData.call.callerIdNumber = string.sub(callData.call.callerIdNumber, 2, -1);
    dataChanged = true;
  end;

  if (callData.call.origin == CallOrigin.Unavailable) then
    callData.call.origin = callDataBasedOrigin(callData);
    dataChanged = true;
  end;

  return dataChanged, callData;
end

-- the caller makes sure that the addressData (the same structure as passed to patchAddressData) is read only
-- if true is returned the number (second returned value) will be used for dialing
function patchDialableNumber(number, addressData)
  if ("string" ~= type(number)) then
    return nil, string.format("number is not string (it's [%s])!", type(number));
  end;
  if ("table" ~= type(addressData)) then
    return nil, string.format("addressData to process is not table (it's [%s])!", type(addressData));
  end;

  local dataChanged = false;
  -- change fields here if necessary (and set dataChanged if so)
  --
  -- Calling ->External numbers needs a 0 prefix
  if (isExternalNumber(number)) then
    number = '0'..number;
    dataChanged = true;
  end;

  return dataChanged, number;
end

-- the caller makes sure that the callData (the same structure as passed to patchCallData) is read only
-- if true is returned the number (second returned value) will be used for transfer
function patchTransferNumber(number, callData)
  --~ if ("string" ~= type(number)) then
    --~ return nil, string.format("number is not string (it's [%s])!", type(number));
  --~ end;
  --~ if ("table" ~= type(callData)) then
    --~ return nil, string.format("callData to process is not table (it's [%s])!", type(callData));
  --~ end;

  local dataChanged = false;
  -- change fields here if necessary (and set dataChanged if so)
  --

  return dataChanged, number;
end
